% CU_GMM.PRG 

% Continuous Updating GMM Estimation and Testing and Confidence Interval Program

% written by:

% Bruce E. Hansen
% Department of Economics
% Social Science Building
% University of Wisconsin
% Madison, WI 53706-1393
% bhansen@ssc.wisc.edu
% http://www.ssc.wisc.edu/~bhansen/

% This program calculates continuously-updated GMM estimates and tests of nonlinear hypotheses in GMM-type moment models.
% It also constructs profile confidence intervals.

% The GMM criterion is optimized using the CONSTRAINED OPTIMIZATION package.  
% The CO and PGRACH libraries must be active.  To do so, type "library co,pgraph" at the Gauss prompt. 

% The user needs to provide a procedure "_moments" which computes the moments,
% and provide a vector of starting values "_beta0".

% To test a hypothesis, the user needs to provide a procedure "h_test" which is a (nonlinear) function
% of the parameter b, which is a zero vector under the null hypothesis.

% To construct a confidence interval for a parameter of interest, the use needs to provide a procedure "h_theta" which
% computes this parameter from the entire parameter vector.

% Format of Procedures:

% m=moments_(b);
% Input: 	b	kx1 parameter vector
% Output:	m	nxm matrix of "moment conditions".  Rows (columns) correspond to observations (moment conditions).
% Example:		proc _moments(b); retp(x.*(y-z*b)); endp;

% h=h_test(b);
% Input: 	b	kx1 parameter vector
% Output:	h	rx1 vector of constraints.  Should be zero under H0
% Example:		proc h_test(b); retp(b[1]+b[2]-2); endp;

% theta=h_theta(b);
% Input: 	b	kx1 parameter vector
% Output:	theta	1x1 parameter
% Example:		proc h_theta(b); retp(b[1]+b[2]); endp;

% beta0_= kx1 	initial value for parameter vector


% The user may also select the following :   **************/
function main()
global y;
global z;
global x;

global center_;center_=1;		% Set to 1 to recenter moments before calculation of weight/variance matrices
		        % else set to 0 @
global testh0_;testh0_=1;		% Set to 1 to test the hypothesis in "h_test", else set to 0 to not do test @
out=fopen('cuout.txt','wt');	
global graph_;graph_=1;		% Set to 1 to compute and display profile criterion, else set to 0 @
global cregion_;;cregion_=1;	    % Set to 1 to compute confidence region for "theta" @
global level_;level_=.90;		% Level of confidence region, if computed @
global grid_;grid_=21;		% Number of gridpoints for graph of profile criterion, if computed @

% Example data set %
load el.txt;
global y; y=el(:,1);
global z; z=el(:,2:3);
global x; x=el(:,4:7);
global testh__;
global region__;

beta0_=(((z'*x)*inv(x'*x)*(x'*y))'/((z'*x)*inv(x'*x)*(x'*z))')';

e=moments_(beta0_);
global m;global n;global k;global fixalpha;
[n,m]=size(e);k=length(beta0_);
global lambda;
names_='Beta1__';
for i=2:k
    names_=[names_;strcat('Beta',num2str(i),'__')];
end;

option=optimset('TolCon',0.001,'TolFun',0.001);
j_gmm_(beta0_);
[beta,j0,ret]=fminunc(@j_gmm_,beta0_);

e=moments_(beta);
g=grads_moment(beta);
if center_==1;
    e = e-ones(length(e(:,1)),1)*mean(e);
end;
v=inv(g'*inv(e'*e)*g);
se=sqrt(diag(v));

fprintf(out,'Continuously-Update GMM Estimation\n\n');
fprintf(out,'Unconstrained Estimation\n\n');
fprintf(out,'Variable     Estimate     St Error\n');
for i=1:length(beta)
    fprintf(out,'%s   %f   %f\n',names_(i,:),beta(i),se(i));
end;
fprintf(out,'J Test of Over-Identifying Restrictions\n');
fprintf(out,'Test     dof     P_Value\n');
fprintf(out,'%f   %f   %f\n',j0,(m-k),1-chi2cdf(j0,m-k));

if testh0_==1
    h=h_test(beta);
    r=length(h(:,1));
    hh=gradh_test(beta)';
    seh=sqrt(diag(hh'*v*hh));

    [beta1,j1,ret]=fmincon(@j_gmm_,beta,[],[],[],[],[],[],@nonlconh_beta);
    
    e=moments_(beta1);
    g=grads_moment(beta);
    if center_==1;
        e = e-ones(length(e(:,1)),1)*mean(e);
    end;
    v1=inv(g'*inv(e'*e)*g);
    hh=gradh_test(beta1)';
    v2=v1-v1*hh*inv(hh'*v1*hh)*(hh'*v1);
    se=sqrt(diag(v2));
    dm=j1-j0;
    
    fprintf(out,'Estimated value of h(beta), s.e. s');
    for i=1:length(h)
        fprintf(out,'%f   %f\n',h(i),seh(i));
    end;
    fprintf(out,'**************************************************\n\n');
    fprintf(out,'Constrained Estimation\n\n');
    fprintf(out,'Variable     Estimate     St Error\n');
    for i=1:length(beta1)
        fprintf(out,'%s   %f   %f\n',names_(i,:),beta1(i),se(i));
    end;
    fprintf(out,'J Test of Over-Identifying Restrictions\n');
    fprintf(out,'Test     dof     P_Value\n');
    fprintf(out,'%f   %f   %f\n',j1,m-k+r,1-chi2cdf(j1,m-k+r));
    fprintf(out,'GMM Distance Test of H0\n');
    fprintf(out,'Test     dof     P_Value\n');
    fprintf(out,'%f   %f   %f\n',dm,r,1-chi2cdf(dm,r));
end;
beta
if (graph_==1)+(cregion_==1)
    thetahat=h_theta(beta);
    hh=gradh_theta(beta)';
    setheta=sqrt(diag(hh'*v*hh));
    if graph_==1;
        [c1,c2]=graph_el(thetahat,setheta,beta);
    else;
        c1=c_solve(thetahat,setheta,beta,1);
        c2=c_solve(thetahat,setheta,beta,2);
    end;
    fprintf(out,'Estimates and Confidence Interval for Parameter of Interest\n');
    fprintf(out,'Theta     s.e.     C1     C2\n');
    fprintf(out,'%f   %f   %f    %f\n',thetahat,setheta,c1,c2);
end;
fclose(out);
    
%********************************************************************
%PROCS
%********************************************************************/
function e=moments_(b);
global y;
global z;
global x;
e=x.*((y-z*b)*ones(1,length(x(1,:))));

% Test of Non-Linear Restrictions @
function h=h_test(b);
global testh0_;
if testh0_==1;
    h=b(1)+b(2)-2;
else;
    h=NaN;
end;

function g=gradh_test(beta);
tiny=0.00001;
p=h_test(beta);
for i=1:length(beta)
    beta1=beta;beta1(i)=beta(i)-tiny;
    beta2=beta;beta2(i)=beta(i)+tiny;
    if i==1
        g=((h_test(beta2)-h_test(beta1))/(2*tiny))';
    else
        g=[g,((h_test(beta2)-h_test(beta1))/(2*tiny))'];
    end;
end;
% Parameter of Interest: Theta @
function h=h_theta(b);
global graph_;
global cregion_;
if (graph_==1)+(cregion_==1);
    h=b(1)+b(2);
else;
    h=NaN;
end;

function g=gradh_theta(beta);
tiny=0.00001;
p=h_theta(beta);
for i=1:length(beta)
    beta1=beta;beta1(i)=beta(i)-tiny;
    beta2=beta;beta2(i)=beta(i)+tiny;
    if i==1
        g=((h_theta(beta2)-h_theta(beta1))/(2*tiny))';
    else
        g=[g,((h_theta(beta2)-h_theta(beta1))/(2*tiny))'];
    end;
end;

function j=j_gmm_(b);
global k;
global center_;
e=moments_(b(1:k));
g=sum(e)';
ee=e'*e;
if center_==1;
    ee=ee-(g*(g'))/length(e);
end;
j=g'*(g'/ee')';

function v=s_moment(b);
v=sum(moments_(b))';

function g=grads_moment(beta);
tiny=0.00001;
p=s_moment(beta);
for i=1:length(beta)
    beta1=beta;beta1(i)=beta(i)-tiny;
    beta2=beta;beta2(i)=beta(i)+tiny;
    if i==1
        g=((s_moment(beta2)-s_moment(beta1))'/(2*tiny))';
    else
        g=[g,((s_moment(beta2)-s_moment(beta1))'/(2*tiny))'];
    end;
end;

function g=p_mo_(beta);
global k;
g=h_theta(beta(1:k))-beta(k+1);

function [l,aa,ret]=constr_(theta,alpha);
global fixalpha;
fixalpha=[alpha;theta];
r=length(alpha(:,1));
r
%co_Active_=[ones(r,1);0];
warning off;
option=optimset('TolFun',0.00000000000000001);
[a,l,ret]=fmincon(@j_gmm_,[alpha;theta],[],[],[],[],[],[],@fnonlcon,option);
warning on;
aa=a(1:r);

function [c,ce] = nonlconh_beta(x)
c=[];
ce=h_test(x);

function [c1,c2]=graph_el(thetahat,setheta,alpha0)
global level_;
global grid_;
cr=chi2inv(level_,1);
a0=alpha0;
lr=0;j=0;
l0=j_gmm_(alpha0);
while lr < (cr+1)

    cr+1
    j=j+1;
    theta0=thetahat-setheta*j;
    theta0
    [l1,a0,ret]=constr_(theta0,a0);
    lr=2*(l1-l0);
    lr
    j
end;
theta1=theta0;
a0=alpha0;
lr=0;j=0;
while lr < (cr+1)
    lr
    cr+1
    j=j+1;
    theta0=thetahat+setheta*j;
    [l1,a0,ret]=constr_(theta0,a0);
    lr=2*(l1-l0);
end;
theta2=theta0;
aa=(theta1:(theta2-theta1)/(grid_-1):theta2);
lrs=zeros(grid_,1);
for i=grid_:-1:1
    i
    theta0=aa(i);
    [l1,a0,ret]=constr_(theta0,a0);
    lr=2*(l1-l0);
    lrs(i)=lr;
end;
z=ones(grid_,1)*cr;
d=(lrs<cr);
d
plot(aa,[lrs,z]);
[temp,d1]=max(d); 
[temp,d2]=max(d((length(d):-1:1)));
d2=grid_+1-d2;

c1a=aa(d1-1);
c1b=aa(d1);
c1=((lrs(d1-1)-cr)*c1b+(cr-lrs(d1))*c1a)/(lrs(d1-1)-lrs(d1));
c2a=aa(d2);
c2b=aa(d2+1);
c2=((lrs(d2+1)-cr)*c2a+(cr-lrs(d2))*c2b)/(lrs(d2+1)-lrs(d2));

lev=num2str(level_*100)
tit1=strcat(lev,'% CU-GMM Confidence Interval for Theta');
tit2=strcat('Profile GMM Criterion and  ',lev,'% Critical Value');
plot(aa,[lrs,z]);
title(tit1);
xlabel('Theta');
ylabel(tit2);
saveas(gcf,'cugmm.eps','psc2');

function theta=c_solve(thetahat,setheta,alpha0,endpoint);
global level_;
global grid_;
gg=0; 
g_l=0;  
theta_l=thetahat; 
g_r=0;  
theta_r=thetahat; 
theta=thetahat;
a0=alpha0;
cr=chi2inv(level_,1);
if endpoint==1; 
    sn=-1; 
elseif endpoint==2; 
    sn=1;
end;
dtheta=sqrt(cr)*setheta*.6*sn;  
l0=j_gmm_(alpha0);
nogo=1;
for j=1:100
    theta=theta+dtheta;
    [l1,a0,ret]=constr_(theta,a0);
    lr=2*(l1-l0);
    if ret>2
        theta=theta-dtheta;
        dtheta=dtheta/2;
        a0=alpha0;
    else;
        gg=lr; 
        theta_l=theta_r;
        g_l=g_r;
        theta_r=theta;
        g_r=gg;
    end;
    try_ =(abs(cr-gg)<.01)+(gg>cr);
    if try_==1;
            nogo=0; 
            break;
    end;
end;
if nogo~=1
    try_=(abs(cr-g_r)<.01)+(abs(theta_l-theta_r)<.000001);
    if try_~=1
        a1=a0; nogo=1;
        for j=1:20
            theta=(theta_l+theta_r)/2;
            [l1,a1,ret]=constr_(theta,a1);
            lr=2*(l1-l0);
            if ret>2;
                [l1,a1,ret]=constr_(theta,a0);
                lr=2*(l1-l0);
            end;
            if lr>cr;
                theta_r=theta; g_r=lr;
            else;
                theta_l=theta; g_l=lr;
            end;
            try_= (abs(cr-lr)<.01)+(abs(theta_l-theta_r)<.000001);
            if try_==1; 
                nogo=0; 
                break; 
            end;
            theta=(theta_l*(g_r-cr)+theta_r*(cr-g_l))/(g_r-g_l);
            [l1,a1,ret]=constr_(theta,a1);
            lr=2*(l1-l0);
            if ret>2;
                [l1,a1,ret]=constr_(theta,a0);
                lr=2*(l1-l0);
            end;
            if lr>cr;
               theta_r=theta; g_r=lr;
            else;
               theta_l=theta; 
               g_l=lr;
            end;
            try_= (abs(cr-lr)<.01)+(abs(theta_l-theta_r)<.000001);
            if try_==1; 
                nogo=0; 
                break; 
            end;
        end; 
    end;
end;

function [c,ce] = fnonlcon(x)
global fixalpha;
r=length(fixalpha);
c=[];
ce=abs(p_mo_(x))+abs(x(r)-fixalpha(r));